@@ -18,36 +18,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Testing the _spatial_mp module."""
 
-
-# try:
-#     from unittest import mock
-# except ImportError:
-#     # separate mock package py<3.3
-#     import mock
 import unittest
+
 import numpy as np
 
 import pyresample._spatial_mp as sp
 
 
-# class SpatialMPTest(unittest.TestCase):
-#     @mock.patch('pyresample._spatial_mp.pyproj.Proj.__init__', return_value=None)
-#     def test_base_proj_epsg(self, proj_init):
-#         """Test Proj creation with EPSG codes"""
-#         if pyproj.__version__ < '2':
-#             return self.skipTest(reason='pyproj 2+ only')
-#
-#         args = [
-#             [None, {'init': 'EPSG:6932'}],
-#             [{'init': 'EPSG:6932'}, {}],
-#             [None, {'EPSG': '6932'}],
-#             [{'EPSG': '6932'}, {}]
-#         ]
-#         for projparams, kwargs in args:
-#             BaseProj(projparams, **kwargs)
-#             proj_init.assert_called_with(projparams='EPSG:6932', preserve_units=mock.ANY)
-#             proj_init.reset_mock()
-
 class SpatialMPTest(unittest.TestCase):
     """Test of spatial_mp."""
 
@@ -87,15 +64,3 @@ class SpatialMPTest(unittest.TestCase):
         self.assertIs(type(coords_float32[0, 0]), np.float32)
         self.assertIs(type(coords_float[0, 0]), np.float64)
         self.assertTrue(np.issubdtype(coords_int.dtype, np.floating))
-
-
-def suite():
-    """Test suite for _spatial_mp."""
-    loader = unittest.TestLoader()
-    mysuite = unittest.TestSuite()
-    mysuite.addTest(loader.loadTestsFromTestCase(SpatialMPTest))
-    return mysuite
-
-
-if __name__ == '__main__':
-    unittest.main()
