@@ -24,7 +24,7 @@ import warnings
 import numpy as np
 
 from pyresample import geometry, kd_tree
-from pyresample.test.utils import catch_warnings
+from pyresample.test.utils import TEST_FILES_PATH, catch_warnings
 
 warnings.simplefilter("always")
 
@@ -32,8 +32,7 @@ warnings.simplefilter("always")
 class Test(unittest.TestCase):
     """Tests for swath definitions."""
 
-    filename = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                                            'test_files', 'ssmis_swath.npz'))
+    filename = os.path.abspath(os.path.join(TEST_FILES_PATH, 'ssmis_swath.npz'))
     data = np.load(filename)['data']
     lons = data[:, 0].astype(np.float64)
     lats = data[:, 1].astype(np.float64)
